#!/usr/bin/python3
import json

import requests

from .dingo_param import (
    CheckClintParam,
    CheckCreateIndexParam,
    CheckVectorAddParam,
    CheckVectorDeleteParam,
    CheckVectorGetParam,
    CheckVectorScanParam,
    CheckVectorSearchParam,
    auto_value_type,
)


class DingoDB:
    headers = {"Content-Type": "application/json"}
    requestProto = "http://"
    indexApi = "/index/api/DINGO/"
    vectorApi = "/vector/api/DINGO/"

    def __init__(self, user: str, password: str, host: list) -> None:
        """
        __init__ init DingoDB

        Args:
            user (str): DingoDB user
            password (str): DingoDB password
            host (list): DingoDB host:port
        """
        params = CheckClintParam(user=user, password=password, host=host)
        self.user = params.user
        self.password = params.password
        self.host = params.host
        self.session = requests.Session()

    def __del__(self):
        self.session.close()

    def close(self):
        self.session.close()

    def make_response(self, res, deal="default"):
        res_json = res.json()
        if res_json.get("status") == 200:
            if res_json.get("message") != "OK":
                return res_json
            return res_json.get("data")
        else:
            raise RuntimeError(res_json)

    def describe_index_info(self, index_name: str) -> dict:
        """
        describe_index_info index info

        Args:
            index_name (str): the name the index

        Raises:
            RuntimeError: return error

        Returns:
            dict: index info
        """
        res = self.session.get(
            f"{self.requestProto}{self.host[0]}{self.indexApi}{index_name}",
            headers=self.headers,
        )
        return self.make_response(res)

    def create_index(
        self,
        index_name: str,
        dimension: int,
        index_type: str = "hnsw",
        metric_type: str = "cosine",
        replicas: int = 3,
        index_config: dict = None,
        metadata_config: dict = None,
        partition_rule: dict = None,
        operand: list = None,
        auto_id: bool = True,
        start_id: int = 1,
    ) -> bool:
        """
        create_index create index

        Args:
            index_name (str): the name of index
            dimension (int): dimension of vector
            index_type (str, optional): index type, one of {"flat", "hnsw","ivf_flat", "ivf_pq", "brute"}. Defaults to "hnsw".
            metric_type (str, optional): metric type, one of {"dotproduct", "euclidean", "cosine"}. Defaults to "cosine"
            replicas (int, optional): dingoDB store replicas. Defaults to 3.
            index_config (dict, optional): Advanced configuration options for the index. Defaults to None.
            metadata_config (dict, optional): metadata. Defaults to None.
            partition_rule (dict, optional): partition rule. Defaults to None.
            operand (list, optional): operand. Defaults to None.
            auto_id (bool, optional): isAutoIncrement or not isAutoIncrement. Defaults to True.
            start_id (int, optional): autoIncrement start id. Defaults to 1.

        Raises:
            RuntimeError: return error

        Returns:
            bool: create table status
        """
        params = CheckCreateIndexParam(
            index_name=index_name,
            dimension=dimension,
            index_type=index_type,
            metric_type=metric_type,
            replicas=replicas,
            index_config=index_config,
            metadata_config=metadata_config,
            partition_rule=partition_rule,
            operand=operand,
            auto_id=auto_id,
            start_id=start_id,
        )

        if params.partition_rule == {} and operand is not None and len(operand) != 0:
            details = []
            for item in operand:
                details.append({"operand": [item], "operator": "", "partName": ""})
            partition_rule = {"cols": [], "details": details, "funcName": ""}

        index_definition = {
            "autoIncrement": params.start_id if params.auto_id else 0,
            "isAutoIncrement": "true" if params.auto_id else "false",
            "indexParameter": {
                "indexType": "INDEX_TYPE_VECTOR",
                "vectorIndexParameter": params.index_config,
            },
            "name": params.index_name,
            "indexPartition": partition_rule
            if partition_rule is not None
            else params.partition_rule,
            "replica": params.replicas,
            "version": 0,
        }
        res = self.session.post(
            f"{self.requestProto}{self.host[0]}{self.indexApi}",
            headers=self.headers,
            data=json.dumps(index_definition),
        )

        return self.make_response(res)

    def update_index_max_element(self, index_name: str, max_element: int) -> bool:
        """
        update_index_max_element change index max element

        only for hnsw

        Args:
            index_name (str): the name of index
            max_element (int): max element value

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """

        res = self.session.put(
            f"{self.requestProto}{self.host[0]}{self.indexApi}{index_name}/{max_element}"
        )
        return self.make_response(res)

    def delete_index(self, index_name: str) -> bool:
        """
        delete_index del/drop index

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        res = self.session.delete(
            f"{self.requestProto}{self.host[0]}{self.indexApi}{index_name}"
        )
        return self.make_response(res)

    def vector_add(
        self, index_name: str, datas: list, vectors: list, ids: list = None
    ) -> list:
        """
        vector_add add vector

        Args:
            index_name (str): the name of index
            datas (list): metadata list
            vectors (list): vector list
            ids (list, optional): id list. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list: add vector info in dingoDB
        """
        params = CheckVectorAddParam(
            index_name=index_name, datas=datas, vectors=vectors, ids=ids
        )

        records = []
        for i, v in enumerate(params.vectors):
            scalar_data = dict(
                (
                    key,
                    {"fieldType": auto_value_type(value), "fields": [{"data": value}]},
                )
                for key, value in params.datas[i].items()
            )
            vector = {
                "binaryValues": [],
                "dimension": len(v),
                "floatValues": v,
                "valueType": "FLOAT",
            }
            record = {"scalarData": scalar_data, "vector": vector}
            if ids is not None:
                record.update({"id": params.ids[i]})

            records.append(record)
        res = self.session.put(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}",
            headers=self.headers,
            data=json.dumps(records),
        )

        return self.make_response(res)

    def vector_upsert(
        self, index_name: str, datas: list, vectors: list, ids: list = None
    ) -> list:
        """
        vector_upsert upsert vector

        Args:
            index_name (str): the name of index
            datas (list): metadata list
            vectors (list): vector list
            ids (list, optional): id list. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list: upsert vector info in dingoDB
        """
        params = CheckVectorAddParam(
            index_name=index_name, datas=datas, vectors=vectors, ids=ids
        )

        records = []
        for i, v in enumerate(params.vectors):
            scalar_data = dict(
                (
                    key,
                    {"fieldType": auto_value_type(value), "fields": [{"data": value}]},
                )
                for key, value in params.datas[i].items()
            )
            vector = {
                "binaryValues": [],
                "dimension": len(v),
                "floatValues": v,
                "valueType": "FLOAT",
            }
            record = {"scalarData": scalar_data, "vector": vector}
            if ids is not None:
                record.update({"id": params.ids[i]})

            records.append(record)
        res = self.session.put(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}/upsert",
            headers=self.headers,
            data=json.dumps(records),
        )

        return self.make_response(res)

    def vector_count(self, index_name: str):
        """
        vector_count count in index

        Args:
            index_name (str): the name of in index

        Returns:
            int: count num
        """
        res = self.session.get(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{index_name}/count",
            headers=self.headers,
        )

        return self.make_response(res)

    def vector_metrics(self, index_name: str):
        """
        vector_metrics metrics in index

        Args:
            index_name (str): the name of in index

        Returns:
            dict: metrics info
        """
        res = self.session.get(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{index_name}",
            headers=self.headers,
        )

        return self.make_response(res)

    def vector_scan(
        self,
        index_name: str,
        start_id: int,
        max_count: int = 1000,
        is_reverse: bool = False,
        with_scalar_data: bool = True,
        with_table_data: bool = True,
        with_vector_data: bool = True,
        fields: list = None,
        filter_scalar: dict = None,
        end_id: int = 0,
    ) -> list:
        """
        vector_scan scan with start_id

        Args:
            index_name (str): the name of in index
            start_id (int): start id
            max_count (int, optional): max scan count. Defaults to 1000.
            is_reverse (bool, optional): whether or not reverse. Defaults to False.
            with_scalar_data (bool, optional): whether  with scalar info. Defaults to True.
            with_table_data (bool, optional): whether  with table info. Defaults to True.
            with_vector_data (bool, optional): whether with vector info. Defaults to True.
            fields (list, optional): fields for return . Defaults to None.
            filter_scalar (dict, optional): filter_scalar for return . Defaults to None.
            end_id (int, optional): if end_id=0, get all max_count . Defaults to 0.

        Raises:
            RuntimeError: return error

        Returns:
            list:  scan info list
        """
        params = CheckVectorScanParam(
            index_name=index_name,
            start_id=start_id,
            max_count=max_count,
            is_reverse=is_reverse,
            without_scalar_data=not with_scalar_data,
            without_table_data=not with_table_data,
            without_vector_data=not with_vector_data,
            fields=fields,
            filter_scalar=filter_scalar,
            end_id=end_id,
        )
        payload = {
            "isReverseScan": params.is_reverse,
            "maxScanCount": params.max_count,
            "scalarForFilter": params.filter_scalar,
            "useScalarFilter": "true" if params.filter_scalar else "false",
            "selectedKeys": params.fields,
            "startId": params.start_id,
            "withoutScalarData": params.without_scalar_data,
            "withoutTableData": params.without_table_data,
            "withoutVectorData": params.without_vector_data,
            "endId": params.end_id,
        }
        res = self.session.post(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}/scan",
            headers=self.headers,
            data=json.dumps(payload),
        )

        return self.make_response(res)

    def get_index(self):
        """
        get_index get all index

        Raises:
            RuntimeError: return error

        Returns:
            list: all index list
        """

        res = self.session.get(
            f"{self.requestProto}{self.host[0]}{self.indexApi}", headers=self.headers
        )

        return self.make_response(res)

    def get_max_index_row(self, index_name: str):
        """
        get_max_index_row get max id in index

        Args:
            index_name (str): the name of in index

        Raises:
            RuntimeError: return error

        Returns:
            int: max id value
        """
        payload = {"isGetMin": "false"}

        res = self.session.get(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{index_name}/id",
            params=payload,
            headers=self.headers,
        )

        return self.make_response(res)

    def vector_search(
        self,
        index_name: str,
        xq: list,
        top_k: int = 10,
        search_params: dict = None,
        pre_filter: bool = True,
        brute: bool = False,
    ) -> dict:
        """
        vector_search search vector

        Args:
            index_name (str): the name of the index
            xq (list): query vector, List[float] or List[List[float]]
            top_k (int, optional): top k search. Defaults to 10.
            search_params (dict, optional): search params for index. Defaults to None.
            pre_filter (bool, optional): filter type for index, False is post-filter. Defaults to True.
            brute (bool, optional): whether to turn on brute force search. Defaults to True.

        Raises:
            RuntimeError: return error

        Returns:
            List[dict]: search results
        """
        index_info = self.describe_index_info(index_name)
        index_type = index_info["indexParameter"]["vectorIndexParameter"][
            "vectorIndexType"
        ]
        params = CheckVectorSearchParam(
            index_name=index_name,
            xq=xq,
            index_type=index_type,
            top_k=top_k,
            pre_filter=pre_filter,
            search_params=search_params,
            brute=brute,
        )
        res = self.session.post(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}",
            headers=self.headers,
            data=json.dumps(params.search_params),
        )

        return self.make_response(res)

    def vector_get(
        self, index_name: str, ids: list, scalar: bool = True, vector: bool = True
    ) -> list:
        """
        vector_get query vector

        Args:
            index_name (str): the name of the index
            ids (list): query id list
            scalar (bool, optional): res with or without scalar. Defaults to True.
            vector (bool, optional): res with or without vector. Defaults to True.

        Raises:
            RuntimeError: _description_

        Returns:
            list: _description_
        """
        params = CheckVectorGetParam(
            index_name=index_name, ids=ids, scalar=scalar, vector=vector
        )
        payload = {
            "ids": params.ids,
            "keys": [],
            "withoutScalarData": "false" if scalar else "true",
            "withoutVectorData": "false" if vector else "true",
        }

        res = self.session.post(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}/get",
            headers=self.headers,
            data=json.dumps(payload),
        )

        return self.make_response(res)

    def vector_delete(self, index_name: str, ids: list):
        """
        vector_delete delete vector with ids

        Args:
            index_name (str): the name of the index
            ids (list): id list

        Raises:
            RuntimeError: return error

        Returns:
            list : [True, False, ...]
        """
        params = CheckVectorDeleteParam(index_name=index_name, ids=ids)

        res = self.session.delete(
            f"{self.requestProto}{self.host[0]}{self.vectorApi}{params.index_name}",
            headers=self.headers,
            data=json.dumps(params.ids),
        )
        return self.make_response(res)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proxy_index.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import proxy_common_pb2 as proxy__common__pb2
import proxy_error_pb2 as proxy__error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11proxy_index.proto\x12\x16\x64ingodb.pb.proxy.index\x1a\x12proxy_common.proto\x1a\x11proxy_error.proto\"\x9f\x01\n\x10VectorAddRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x36\n\x07vectors\x18\x03 \x03(\x0b\x32%.dingodb.pb.proxy.common.VectorWithId\x12\x17\n\x0freplace_deleted\x18\x04 \x01(\x08\x12\x11\n\tis_update\x18\x05 \x01(\x08\"y\n\x11VectorAddResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\x36\n\x07vectors\x18\x02 \x03(\x0b\x32%.dingodb.pb.proxy.common.VectorWithId\"\xbc\x01\n\x10VectorGetRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x12\n\nvector_ids\x18\x03 \x03(\x03\x12\x1b\n\x13without_vector_data\x18\x04 \x01(\x08\x12\x1b\n\x13without_scalar_data\x18\x05 \x01(\x08\x12\x15\n\rselected_keys\x18\x06 \x03(\t\x12\x1a\n\x12without_table_data\x18\x07 \x01(\x08\"y\n\x11VectorGetResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\x36\n\x07vectors\x18\x02 \x03(\x0b\x32%.dingodb.pb.proxy.common.VectorWithId\"\xb9\x01\n\x13VectorSearchRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x36\n\x07vectors\x18\x03 \x03(\x0b\x32%.dingodb.pb.proxy.common.VectorWithId\x12\x41\n\tparameter\x18\x04 \x01(\x0b\x32..dingodb.pb.proxy.common.VectorSearchParameter\"f\n\x18VectorWithDistanceResult\x12J\n\x15vector_with_distances\x18\x01 \x03(\x0b\x32+.dingodb.pb.proxy.common.VectorWithDistance\"\x8d\x01\n\x14VectorSearchResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12G\n\rbatch_results\x18\x02 \x03(\x0b\x32\x30.dingodb.pb.proxy.index.VectorWithDistanceResult\"K\n\x13VectorDeleteRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x0b\n\x03ids\x18\x03 \x03(\x03\"X\n\x14VectorDeleteResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\x12\n\nkey_states\x18\x02 \x03(\x08\"T\n\x18VectorGetBorderIdRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x0f\n\x07get_min\x18\x03 \x01(\x08\"U\n\x19VectorGetBorderIdResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\n\n\x02id\x18\x02 \x01(\x03\"\xf0\x02\n\x16VectorScanQueryRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x17\n\x0fvector_id_start\x18\x03 \x01(\x03\x12\x17\n\x0fis_reverse_scan\x18\x04 \x01(\x08\x12\x16\n\x0emax_scan_count\x18\x05 \x01(\x03\x12\x15\n\rvector_id_end\x18\x06 \x01(\x03\x12\x1b\n\x13without_vector_data\x18\x0b \x01(\x08\x12\x1b\n\x13without_scalar_data\x18\x0c \x01(\x08\x12\x15\n\rselected_keys\x18\r \x03(\t\x12\x1a\n\x12without_table_data\x18\x0e \x01(\x08\x12\x19\n\x11use_scalar_filter\x18\x14 \x01(\x08\x12\x44\n\x11scalar_for_filter\x18\x15 \x01(\x0b\x32).dingodb.pb.proxy.common.VectorScalarData\"\x7f\n\x17VectorScanQueryResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\x36\n\x07vectors\x18\x02 \x03(\x0b\x32%.dingodb.pb.proxy.common.VectorWithId\"H\n\x1dVectorGetRegionMetricsRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\"\x8c\x01\n\x1eVectorGetRegionMetricsResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12<\n\x07metrics\x18\x02 \x01(\x0b\x32+.dingodb.pb.proxy.common.VectorIndexMetrics\"=\n\x12VectorCountRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\"R\n\x13VectorCountResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\r\n\x05\x63ount\x18\x02 \x01(\x03\x32\xed\x07\n\x0cIndexService\x12`\n\tVectorAdd\x12(.dingodb.pb.proxy.index.VectorAddRequest\x1a).dingodb.pb.proxy.index.VectorAddResponse\x12`\n\tVectorGet\x12(.dingodb.pb.proxy.index.VectorGetRequest\x1a).dingodb.pb.proxy.index.VectorGetResponse\x12\x63\n\x0cVectorUpsert\x12(.dingodb.pb.proxy.index.VectorAddRequest\x1a).dingodb.pb.proxy.index.VectorAddResponse\x12i\n\x0cVectorSearch\x12+.dingodb.pb.proxy.index.VectorSearchRequest\x1a,.dingodb.pb.proxy.index.VectorSearchResponse\x12i\n\x0cVectorDelete\x12+.dingodb.pb.proxy.index.VectorDeleteRequest\x1a,.dingodb.pb.proxy.index.VectorDeleteResponse\x12x\n\x11VectorGetBorderId\x12\x30.dingodb.pb.proxy.index.VectorGetBorderIdRequest\x1a\x31.dingodb.pb.proxy.index.VectorGetBorderIdResponse\x12r\n\x0fVectorScanQuery\x12..dingodb.pb.proxy.index.VectorScanQueryRequest\x1a/.dingodb.pb.proxy.index.VectorScanQueryResponse\x12\x87\x01\n\x16VectorGetRegionMetrics\x12\x35.dingodb.pb.proxy.index.VectorGetRegionMetricsRequest\x1a\x36.dingodb.pb.proxy.index.VectorGetRegionMetricsResponse\x12\x66\n\x0bVectorCount\x12*.dingodb.pb.proxy.index.VectorCountRequest\x1a+.dingodb.pb.proxy.index.VectorCountResponseB\x1b\n\x16io.dingodb.proxy.index\x88\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proxy_index_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026io.dingodb.proxy.index\210\001\001'
  _globals['_VECTORADDREQUEST']._serialized_start=85
  _globals['_VECTORADDREQUEST']._serialized_end=244
  _globals['_VECTORADDRESPONSE']._serialized_start=246
  _globals['_VECTORADDRESPONSE']._serialized_end=367
  _globals['_VECTORGETREQUEST']._serialized_start=370
  _globals['_VECTORGETREQUEST']._serialized_end=558
  _globals['_VECTORGETRESPONSE']._serialized_start=560
  _globals['_VECTORGETRESPONSE']._serialized_end=681
  _globals['_VECTORSEARCHREQUEST']._serialized_start=684
  _globals['_VECTORSEARCHREQUEST']._serialized_end=869
  _globals['_VECTORWITHDISTANCERESULT']._serialized_start=871
  _globals['_VECTORWITHDISTANCERESULT']._serialized_end=973
  _globals['_VECTORSEARCHRESPONSE']._serialized_start=976
  _globals['_VECTORSEARCHRESPONSE']._serialized_end=1117
  _globals['_VECTORDELETEREQUEST']._serialized_start=1119
  _globals['_VECTORDELETEREQUEST']._serialized_end=1194
  _globals['_VECTORDELETERESPONSE']._serialized_start=1196
  _globals['_VECTORDELETERESPONSE']._serialized_end=1284
  _globals['_VECTORGETBORDERIDREQUEST']._serialized_start=1286
  _globals['_VECTORGETBORDERIDREQUEST']._serialized_end=1370
  _globals['_VECTORGETBORDERIDRESPONSE']._serialized_start=1372
  _globals['_VECTORGETBORDERIDRESPONSE']._serialized_end=1457
  _globals['_VECTORSCANQUERYREQUEST']._serialized_start=1460
  _globals['_VECTORSCANQUERYREQUEST']._serialized_end=1828
  _globals['_VECTORSCANQUERYRESPONSE']._serialized_start=1830
  _globals['_VECTORSCANQUERYRESPONSE']._serialized_end=1957
  _globals['_VECTORGETREGIONMETRICSREQUEST']._serialized_start=1959
  _globals['_VECTORGETREGIONMETRICSREQUEST']._serialized_end=2031
  _globals['_VECTORGETREGIONMETRICSRESPONSE']._serialized_start=2034
  _globals['_VECTORGETREGIONMETRICSRESPONSE']._serialized_end=2174
  _globals['_VECTORCOUNTREQUEST']._serialized_start=2176
  _globals['_VECTORCOUNTREQUEST']._serialized_end=2237
  _globals['_VECTORCOUNTRESPONSE']._serialized_start=2239
  _globals['_VECTORCOUNTRESPONSE']._serialized_end=2321
  _globals['_INDEXSERVICE']._serialized_start=2324
  _globals['_INDEXSERVICE']._serialized_end=3329
# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3

from dingodb.sdk_client import SDKClient
from dingodb.sdk_vector.sdk_vector_client import SDKVectorClient

from dingodb.common.vector_rep import ScalarSchema, RegionState, RegionStatus

from .sdk_vector_param import (
    CreateIndexParam,
    VectorAddParam,
    VectorScanParam,
    VectorSearchParam,
    VectorGetParam,
    VectorDeleteParam,
    VectorStatusByRegionIdParam,
    VectorBuildByRegionIdParam,
    VectorLoadByRegionIdParam,
    VectorResetByRegionIdParam,
)


class SDKVectorDingoDB:
    def __init__(self, client: SDKClient):
        """
        __init__ init DingoSDK

        Args:
            client: SDKClient
        """
        self.client = SDKVectorClient(client)

    def describe_index_info(self, index_name: str) -> dict:
        """
        describe_index_info index info

        Args:
            index_name (str): the name the index

        Raises:
            RuntimeError: return error

        Returns:
            dict: index info
        """
        raise RuntimeError("not implement")

    def describe_index_info_all(self) -> dict:
        """
        describe_index_info index info

        Raises:
            RuntimeError: return error

        Returns:
            List[dict]: index info
        """
        raise RuntimeError("not implement")

    def create_index(
        self,
        index_name: str,
        dimension: int,
        index_type: str = "hnsw",
        metric_type: str = "cosine",
        replicas: int = 3,
        index_config: dict = None,
        metadata_config: dict = None,
        partition_rule: dict = None,
        operand: list = None,
        auto_id: bool = True,
        start_id: int = 1,
    ) -> bool:
        """
        create_index create index

        Args:
            index_name (str): the name of index
            dimension (int): dimension of vector
            index_type (str, optional): index type, one of {"flat", "hnsw","ivf_flat", "ivf_pq", "brute"}. Defaults to "hnsw".
            metric_type (str, optional): metric type, one of {"dotproduct", "euclidean", "cosine"}. Defaults to "cosine"
            replicas (int, optional): dingoDB store replicas. Defaults to 3.
            index_config (dict, optional): Advanced configuration options for the index. Defaults to None.
            metadata_config (dict, optional): metadata. Defaults to None.NOT Support Now , used for schema
            partition_rule (dict, optional): partition rule. Defaults to None. NOT Support Now
            operand (list, optional): operand. Defaults to None.
            auto_id (bool, optional): isAutoIncrement or not isAutoIncrement. Defaults to True.
            start_id (int, optional): autoIncrement start id. Defaults to 1.

        Raises:
            RuntimeError: return error

        Returns:
            bool: create table status
        """
        params = CreateIndexParam(
            index_name=index_name,
            dimension=dimension,
            index_type=index_type,
            metric_type=metric_type,
            replicas=replicas,
            index_config=index_config,
            metadata_config=metadata_config,
            partition_rule=partition_rule,
            operand=operand,
            auto_id=auto_id,
            start_id=start_id,
        )

        return self.client.create_index(params)

    def create_index_with_schema(
        self,
        index_name: str,
        dimension: int,
        schema: ScalarSchema,
        index_type: str = "hnsw",
        metric_type: str = "cosine",
        replicas: int = 3,
        index_config: dict = None,
        metadata_config: dict = None,
        partition_rule: dict = None,
        operand: list = None,
        auto_id: bool = True,
        start_id: int = 1,
    ) -> bool:
        """
        create_index create index

        Args:
            index_name (str): the name of index
            dimension (int): dimension of vector
            schema (ScalarSchema): schema of scalar
            index_type (str, optional): index type, one of {"flat", "hnsw","ivf_flat", "ivf_pq", "brute","binary_flat","binary_ivf_flat"}. Defaults to "hnsw".
            metric_type (str, optional): metric type, one of {"dotproduct", "euclidean", "cosine","hamming"}. Defaults to "cosine"
            replicas (int, optional): dingoDB store replicas. Defaults to 3.
            index_config (dict, optional): Advanced configuration options for the index. Defaults to None.
            metadata_config (dict, optional): metadata. Defaults to None.NOT Support Now , used for schema
            partition_rule (dict, optional): partition rule. Defaults to None. NOT Support Now
            operand (list, optional): operand. Defaults to None.
            auto_id (bool, optional): isAutoIncrement or not isAutoIncrement. Defaults to True.
            start_id (int, optional): autoIncrement start id. Defaults to 1.

        Raises:
            RuntimeError: return error

        Returns:
            bool: create table status
        """
        params = CreateIndexParam(
            index_name=index_name,
            dimension=dimension,
            index_type=index_type,
            metric_type=metric_type,
            replicas=replicas,
            index_config=index_config,
            metadata_config=metadata_config,
            partition_rule=partition_rule,
            operand=operand,
            auto_id=auto_id,
            start_id=start_id,
        )

        return self.client.create_index(param=params, schema=schema)

    def update_index_max_element(self, index_name: str, max_element: int) -> bool:
        """
        update_index_max_element change index max element

        only for hnsw

        Args:
            index_name (str): the name of index
            max_element (int): max element value

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        return True

    def update_index(self, index_name: str, definition: dict) -> bool:
        """
        update_index_max_element change index max element

        only for hnsw

        Args:
            index_name (str): the name of index
            max_element (int): max element value

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        return True

    def delete_index(self, index_name: str) -> bool:
        """
        delete_index del/drop index

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        return self.client.delete_index(index_name)

    # TODO: only return vector ids
    def vector_add(
        self, index_name: str, datas: list, vectors: list, ids: list = None,value_type: str="float"
    ) -> list:
        """
        vector_add add vector

        Args:
            index_name (str): the name of index
            datas (list): metadata list
            vectors (list): vector list
            ids (list, optional): id list. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list: vector id list
        """
        params = VectorAddParam(
            index_name=index_name, datas=datas, vectors=vectors, ids=ids,value_type=value_type
        )

        return self.client.vector_add(params)

    def vector_upsert(
        self, index_name: str, datas: list, vectors: list, ids: list = None ,value_type: str="float"
    ) -> list:
        """
        vector_upsert upsert vector

        Args:
            index_name (str): the name of index
            datas (list): metadata list
            vectors (list): vector list
            ids (list, optional): id list. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list: upsert vector info in dingoDB
        """
        params = VectorAddParam(
            index_name=index_name, datas=datas, vectors=vectors, ids=ids,value_type=value_type
        )

        return self.client.vector_upsert(params)

    def vector_count(self, index_name: str):
        """
        vector_count count in index

        Args:
            index_name (str): the name of in index

        Raises:
            RuntimeError: return error

        Returns:
            int: count num
        """
        return self.client.vector_count(index_name)

    def vector_metrics(self, index_name: str):
        """
        vector_metrics metrics in index

        Args:
            index_name (str): the name of in index

        Returns:
            str: metrics string
        """
        return self.client.vector_metrics(index_name)

    def vector_scan(
        self,
        index_name: str,
        start_id: int,
        max_count: int = 1000,
        is_reverse: bool = False,
        with_scalar_data: bool = True,
        with_table_data: bool = True,
        with_vector_data: bool = True,
        fields: list = None,
        filter_scalar: dict = None,
        end_id: int = 0,
    ) -> list:
        """
        vector_scan scan with start_id

        Args:
            index_name (str): the name of in index
            start_id (int): start id
            max_count (int, optional): max scan count. Defaults to 1000.
            is_reverse (bool, optional): whether reverse. Defaults to False.
            with_scalar_data (bool, optional): whether  with scalar info. Defaults to True.
            with_table_data (bool, optional): whether  with table info. Defaults to True.
            with_vector_data (bool, optional): whether with vector info. Defaults to True.
            fields (list, optional): fields for return . Defaults to [].
            filter_scalar (dict, optional): filter_scalar for return . Defaults to None.
            end_id (int, optional): if end_id=0, get all max_count . Defaults to 0.

        Raises:
            RuntimeError: return error

        Returns:
            list:  scan info list
        """
        params = VectorScanParam(
            index_name=index_name,
            start_id=start_id,
            max_count=max_count,
            is_reverse=is_reverse,
            with_scalar_data=with_scalar_data,
            with_table_data=with_table_data,
            with_vector_data=with_vector_data,
            fields=fields,
            filter_scalar=filter_scalar,
            end_id=end_id,
        )

        return self.client.vector_scan(params)

    def get_index(self):
        """
        get_index get all index

        Raises:
            RuntimeError: return error

        Returns:
            list: all index list
        """
        return []

    def get_max_index_row(self, index_name: str):
        """
        get_max_index_row get max id in index

        Args:
            index_name (str): the name of in index

        Raises:
            RuntimeError: return error

        Returns:
            int: max id value
        """
        return self.client.get_max_index_row(index_name=index_name)

    def vector_search(
        self,
        index_name: str,
        xq: list,
        top_k: int = 10,
        search_params: dict = None,
        pre_filter: bool = True,
        brute: bool = False,
        value_type: str = "float",
    ) -> list:
        """
        vector_search search vector

        Args:
            index_name (str): the name of the index
            xq (list): query vector, List[float] or List[List[float]]
            top_k (int, optional): top k search. Defaults to 10.
            search_params (dict, optional): search params for index. Defaults to None.
            pre_filter (bool, optional): filter type for index, False is post-filter. Defaults to True.
            brute (bool, optional): whether to turn on brute force search. Defaults to True.

        Raises:
            RuntimeError: return error

        Returns:
            List[dict]: search results
        """

        params = VectorSearchParam(
            index_name=index_name,
            xq=xq,
            top_k=top_k,
            pre_filter=pre_filter,
            search_params=search_params,
            brute=brute,
            value_type=value_type
        )

        return self.client.vector_search(params)

    def vector_get(
        self, index_name: str, ids: list, scalar: bool = True, vector: bool = True
    ) -> list:
        """
        vector_get query vector

        Args:
            index_name (str): the name of the index
            ids (list): query id list
            scalar (bool, optional): res with or without scalar. Defaults to True.
            vector (bool, optional): res with or without vector. Defaults to True.

        Raises:
            RuntimeError

        Returns:
            list: VectorWithId dict
        """
        params = VectorGetParam(
            index_name=index_name,
            ids=ids,
            with_scalar_data=scalar,
            with_vector_data=vector,
        )

        return self.client.vector_get(params)

    def vector_delete(self, index_name: str, ids: list):
        """
        vector_delete delete vector with ids

        Args:
            index_name (str): the name of the index
            ids (list): id list

        Raises:
            RuntimeError: return error

        Returns:
            list : [True, False, ...]
        """
        params = VectorDeleteParam(index_name=index_name, ids=ids)

        return self.client.vector_delete(params)

    def vector_status_by_index(self, index_name: str) -> list[RegionState]:
        """
        vector_status_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: state list
        """
        return self.client.vector_status_by_index(index_name=index_name)

    def vector_status_by_region(self, index_name: str, ids: list) -> list[RegionState]:
        """
        vector_status_by_region

        Args:
             index_name: str
              ids: list

        Raises:
            RuntimeError: return error

        Returns:
            list: state list
        """
        params = VectorStatusByRegionIdParam(index_name=index_name, ids=ids)
        return self.client.vector_status_by_region(params)

    def vector_build_by_index(self, index_name: str) -> list[RegionStatus]:
        """
        vector_build_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        return self.client.vector_build_by_index(index_name=index_name)

    def vector_build_by_region(self, index_name: str, ids: list) -> list[RegionStatus]:
        """
        vector_build_by_index

        Args:
           index_name: str
            ids: list

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        params = VectorBuildByRegionIdParam(index_name=index_name, ids=ids)
        return self.client.vector_build_by_region(params)

    def vector_load_by_index(self, index_name: str) -> list[RegionStatus]:
        """
        vector_load_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        return self.client.vector_load_by_index(index_name=index_name)

    def vector_load_by_region(self, index_name: str, ids: list) -> list[RegionStatus]:
        """
        vector_load_by_region

        Args:
            index_name: str
             ids: list

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        params = VectorLoadByRegionIdParam(index_name=index_name, ids=ids)
        return self.client.vector_load_by_region(params)

    def vector_reset_by_index(self, index_name: str) -> list[RegionStatus]:
        """
        vector_reset_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        return self.client.vector_reset_by_index(index_name=index_name)

    def vector_reset_by_region(self, index_name: str, ids: list) -> list[RegionStatus]:
        """
        vector_reset_by_region

        Args:
            index_name: str
             ids: list

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        params = VectorResetByRegionIdParam(index_name=index_name, ids=ids)
        return self.client.vector_reset_by_region(params)

    def vector_import_add(
        self, index_name: str, datas: list, vectors: list, ids: list = None
    ) -> list:
        """
        vector_import_add add vector

        Args:
            index_name (str): the name of index
            datas (list): metadata list
            vectors (list): vector list
            ids (list, optional): id list. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list: vector id list
        """
        params = VectorAddParam(
            index_name=index_name, datas=datas, vectors=vectors, ids=ids
        )

        return self.client.vector_import_add(params)

    def vector_import_delete(self, index_name: str, ids: list):
        """
        vector_delete delete vector with ids

        Args:
            index_name (str): the name of the index
            ids (list): id list

        Raises:
            RuntimeError: return error

        Returns:

        """
        params = VectorDeleteParam(index_name=index_name, ids=ids)

        return self.client.vector_import_delete(params)

    def vector_count_memory(self, index_name: str) -> int:
        """
        vector_count_memory  count build vectors

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            count
        """
        return self.client.vector_count_memory(index_name=index_name)

    def vector_get_auto_increment_id(self, index_name: str) -> int:
        """
        vector_get_auto_increment_id get_auto_increment_id

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            auto_increment_id
        """
        return self.client.vector_get_auto_increment_memory(index_name=index_name)

    def vector_update_auto_increment_id(self, index_name: str, start_id: int):
        """
        vector_update_auto_increment_id update AutoIncrementId

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:

        """
        return self.client.vector_update_auto_increment_id(
            index_name=index_name, start_id=start_id
        )
    
    def vector_dump(self, index_name: str)-> list:
        return self.client.vector_dump(index_name=index_name)



#!/usr/bin/python3

import dingosdk

from dingodb.common.vector_rep import (
    ScalarType,
    ScarlarValue,
    ScalarData,
    Vector,
    VectorWithId,
    VectorValueType,
    SearchResult,
    VectorWithDistance,
    MetricType,
    IndexMetric,
    RegionStatus,
    RegionState,
)

from typing import List, Dict

sdk_value_type_to_vector_value_type = {
    dingosdk.ValueType.kFloat: VectorValueType.FLOAT,
    dingosdk.ValueType.kUint8: VectorValueType.BINARY,
}

sdk_type_to_scalar_type = {
    dingosdk.Type.kBOOL: ScalarType.BOOL,
    dingosdk.Type.kINT64: ScalarType.INT64,
    dingosdk.Type.kDOUBLE: ScalarType.DOUBLE,
    dingosdk.Type.kSTRING: ScalarType.STRING,
}

sdk_metric_type_to_metric_type = {
    dingosdk.MetricType.kL2: MetricType.L2,
    dingosdk.MetricType.kInnerProduct: MetricType.INNER_PRODUCT,
    dingosdk.MetricType.kCosine: MetricType.COSINE,
    dingosdk.MetricType.kHamming: MetricType.HAMMING
}


def sdk_scalar_fields_to_fields_list(
    scalar_type: ScalarType, sdk_scalar_fileds: List
) -> list:
    if scalar_type == ScalarType.BOOL:
        return [x.bool_data for x in sdk_scalar_fileds]
    elif scalar_type == ScalarType.INT64:
        return [x.long_data for x in sdk_scalar_fileds]
    elif scalar_type == ScalarType.DOUBLE:
        return [x.double_data for x in sdk_scalar_fileds]
    elif scalar_type == ScalarType.STRING:
        return [x.string_data for x in sdk_scalar_fileds]
    else:
        raise RuntimeError(f"Unknown scalar type {scalar_type}")


def sdk_scalar_value_to_scalar_value(
    sdk_scalar_value: dingosdk.ScalarValue,
) -> ScarlarValue:
    return ScarlarValue(
        scalar_type=sdk_type_to_scalar_type[sdk_scalar_value.type],
        fields=sdk_scalar_fields_to_fields_list(
            sdk_type_to_scalar_type[sdk_scalar_value.type], sdk_scalar_value.fields
        ),
    )


def sdk_scalar_data_to_scalar_data(
    sdk_scalar_data: Dict,
) -> ScalarData:
    scalar_data = ScalarData()
    for key, value in sdk_scalar_data.items():
        scalar_data.add_scalar_value(key, sdk_scalar_value_to_scalar_value(value))

    return scalar_data


def sdk_vector_with_id_to_vector_with_id(
    sdk_vector_with_id: dingosdk.VectorWithId,
) -> VectorWithId:
    return VectorWithId(
        id=sdk_vector_with_id.id,
        vector=Vector(
            dimension=sdk_vector_with_id.vector.dimension,
            value_type=sdk_value_type_to_vector_value_type[
                sdk_vector_with_id.vector.value_type
            ],
            float_values=sdk_vector_with_id.vector.float_values,
            binary_values=sdk_vector_with_id.vector.binary_values,
        ),
        scalar_data=sdk_scalar_data_to_scalar_data(sdk_vector_with_id.scalar_data),
    )


def sdk_vector_with_distance_to_vector_with_distance(
    sdk_vector_with_distance: dingosdk.VectorWithDistance,
) -> VectorWithDistance:
    return VectorWithDistance(
        vector_with_id=sdk_vector_with_id_to_vector_with_id(
            sdk_vector_with_distance.vector_data
        ),
        distance=sdk_vector_with_distance.distance,
        metric_type=sdk_metric_type_to_metric_type[
            sdk_vector_with_distance.metric_type
        ],
    )


def sdk_search_result_to_search_result(
    sdk_search_result: dingosdk.SearchResult,
) -> SearchResult:
    return SearchResult(
        vector_with_distance_list=[
            sdk_vector_with_distance_to_vector_with_distance(x)
            for x in sdk_search_result.vector_datas
        ]
    )


def sdk_index_metrics_result_to_index_metric(
    sdk_index_metrics_result: dingosdk.IndexMetricsResult,
) -> IndexMetric:
    return IndexMetric(
        index_type=dingosdk.VectorIndexTypeToString(
            sdk_index_metrics_result.index_type
        ),
        count=sdk_index_metrics_result.count,
        deleted_count=sdk_index_metrics_result.deleted_count,
        max_vector_id=sdk_index_metrics_result.max_vector_id,
        min_vector_id=sdk_index_metrics_result.min_vector_id,
        memory_bytes=sdk_index_metrics_result.memory_bytes,
    )


def sdk_err_status_result_to_err_status(
    sdk_err_status_result: dingosdk.ErrStatusResult,
) -> list[RegionStatus]:
    err_status: list[RegionStatus] = []
    for e in sdk_err_status_result.region_status:
        region_status = RegionStatus(region_id=e.region_id, err_msg=e.status.ToString())
        err_status.append(region_status)
    return err_status


def sdk_state_result_to_state(sdk_state_result: dingosdk.StateResult) -> list:
    states: list[RegionState] = []
    for s in sdk_state_result.region_states:
        region_state = RegionState(
            region_id=s.region_id,
            state=dingosdk.RegionStateToString(s.state),
        )
        states.append(region_state)

    return states


def type_conversion(sdk_type: dingosdk.Type, sch_type: dingosdk.Type) -> bool:
    if sdk_type == sch_type:
        return True

    kTypeConversionMatrix = [
        [True, False, False, False],  # kBOOL can be converted to kBOOL
        [False, True, True, False],  # kINT64 can be converted to kINT64, kDOUBLE
        [False, False, True, False],  # kDOUBLE can be converted to kDOUBLE
        [False, False, False, True],  # kSTRING can be converted to kSTRING
    ]
    type_to_index = {
        dingosdk.Type.kBOOL: 0,
        dingosdk.Type.kINT64: 1,
        dingosdk.Type.kDOUBLE: 2,
        dingosdk.Type.kSTRING: 3,
    }

    try:
        row = type_to_index[sdk_type]
    except KeyError:
        raise RuntimeError(f"not support sdk_type: {sdk_type}")

    try:
        col = type_to_index[sch_type]
    except KeyError:
        raise RuntimeError(f"not support sch_type: {sch_type}")

    return kTypeConversionMatrix[row][col]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proxy_error.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import proxy_common_pb2 as proxy__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11proxy_error.proto\x12\x16\x64ingodb.pb.proxy.error\x1a\x12proxy_common.proto\"G\n\x05\x45rror\x12.\n\x07\x65rrcode\x18\x01 \x01(\x0e\x32\x1d.dingodb.pb.proxy.error.Errno\x12\x0e\n\x06\x65rrmsg\x18\x02 \x01(\t*\x1f\n\x05\x45rrno\x12\x06\n\x02OK\x10\x00\x12\x0e\n\tEINTERNAL\x10\xf4\x03\x42\x1b\n\x16io.dingodb.proxy.error\x88\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proxy_error_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026io.dingodb.proxy.error\210\001\001'
  _globals['_ERRNO']._serialized_start=138
  _globals['_ERRNO']._serialized_end=169
  _globals['_ERROR']._serialized_start=65
  _globals['_ERROR']._serialized_end=136
# @@protoc_insertion_point(module_scope)



import proxy_common_pb2 as _proxy_common_pb2
import proxy_error_pb2 as _proxy_error_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PartitionDetailDefinition(_message.Message):
    __slots__ = ["part_name", "operator", "operand"]
    PART_NAME_FIELD_NUMBER: _ClassVar[int]
    OPERATOR_FIELD_NUMBER: _ClassVar[int]
    OPERAND_FIELD_NUMBER: _ClassVar[int]
    part_name: str
    operator: str
    operand: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, part_name: _Optional[str] = ..., operator: _Optional[str] = ..., operand: _Optional[_Iterable[str]] = ...) -> None: ...

class PartitionRule(_message.Message):
    __slots__ = ["func_name", "columns", "details"]
    FUNC_NAME_FIELD_NUMBER: _ClassVar[int]
    COLUMNS_FIELD_NUMBER: _ClassVar[int]
    DETAILS_FIELD_NUMBER: _ClassVar[int]
    func_name: str
    columns: _containers.RepeatedScalarFieldContainer[str]
    details: _containers.RepeatedCompositeFieldContainer[PartitionDetailDefinition]
    def __init__(self, func_name: _Optional[str] = ..., columns: _Optional[_Iterable[str]] = ..., details: _Optional[_Iterable[_Union[PartitionDetailDefinition, _Mapping]]] = ...) -> None: ...

class IndexDefinition(_message.Message):
    __slots__ = ["name", "version", "index_partition", "replica", "index_parameter", "with_auto_increment", "auto_increment"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    INDEX_PARTITION_FIELD_NUMBER: _ClassVar[int]
    REPLICA_FIELD_NUMBER: _ClassVar[int]
    INDEX_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    WITH_AUTO_INCREMENT_FIELD_NUMBER: _ClassVar[int]
    AUTO_INCREMENT_FIELD_NUMBER: _ClassVar[int]
    name: str
    version: int
    index_partition: PartitionRule
    replica: int
    index_parameter: _proxy_common_pb2.IndexParameter
    with_auto_increment: bool
    auto_increment: int
    def __init__(self, name: _Optional[str] = ..., version: _Optional[int] = ..., index_partition: _Optional[_Union[PartitionRule, _Mapping]] = ..., replica: _Optional[int] = ..., index_parameter: _Optional[_Union[_proxy_common_pb2.IndexParameter, _Mapping]] = ..., with_auto_increment: bool = ..., auto_increment: _Optional[int] = ...) -> None: ...

class CreateIndexRequest(_message.Message):
    __slots__ = ["schema_name", "definition"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    DEFINITION_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    definition: IndexDefinition
    def __init__(self, schema_name: _Optional[str] = ..., definition: _Optional[_Union[IndexDefinition, _Mapping]] = ...) -> None: ...

class CreateIndexResponse(_message.Message):
    __slots__ = ["error", "state"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    state: bool
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., state: bool = ...) -> None: ...

class UpdateIndexRequest(_message.Message):
    __slots__ = ["schema_name", "definition"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    DEFINITION_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    definition: IndexDefinition
    def __init__(self, schema_name: _Optional[str] = ..., definition: _Optional[_Union[IndexDefinition, _Mapping]] = ...) -> None: ...

class UpdateIndexResponse(_message.Message):
    __slots__ = ["error", "state"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    state: bool
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., state: bool = ...) -> None: ...

class UpdateMaxElementsRequest(_message.Message):
    __slots__ = ["schema_name", "index_name", "max_elements"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    MAX_ELEMENTS_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    max_elements: int
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ..., max_elements: _Optional[int] = ...) -> None: ...

class UpdateMaxElementsResponse(_message.Message):
    __slots__ = ["error", "state"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    state: bool
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., state: bool = ...) -> None: ...

class DeleteIndexRequest(_message.Message):
    __slots__ = ["schema_name", "index_name"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ...) -> None: ...

class DeleteIndexResponse(_message.Message):
    __slots__ = ["error", "state"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    state: bool
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., state: bool = ...) -> None: ...

class GetIndexRequest(_message.Message):
    __slots__ = ["schema_name", "index_name"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ...) -> None: ...

class GetIndexResponse(_message.Message):
    __slots__ = ["error", "definition"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    DEFINITION_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    definition: IndexDefinition
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., definition: _Optional[_Union[IndexDefinition, _Mapping]] = ...) -> None: ...

class GetIndexesRequest(_message.Message):
    __slots__ = ["schema_name"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    def __init__(self, schema_name: _Optional[str] = ...) -> None: ...

class GetIndexesResponse(_message.Message):
    __slots__ = ["error", "definitions"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    DEFINITIONS_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    definitions: _containers.RepeatedCompositeFieldContainer[IndexDefinition]
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., definitions: _Optional[_Iterable[_Union[IndexDefinition, _Mapping]]] = ...) -> None: ...

class GetIndexNamesRequest(_message.Message):
    __slots__ = ["schema_name"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    def __init__(self, schema_name: _Optional[str] = ...) -> None: ...

class GetIndexNamesResponse(_message.Message):
    __slots__ = ["error", "names"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    NAMES_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    names: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., names: _Optional[_Iterable[str]] = ...) -> None: ...

class GetIndexMetricsRequest(_message.Message):
    __slots__ = ["schema_name", "index_name"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ...) -> None: ...

class GetIndexMetricsResponse(_message.Message):
    __slots__ = ["error", "rows_count", "min_key", "max_key", "part_count", "index_type", "current_count", "deleted_count", "max_id", "min_id", "memory_bytes"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ROWS_COUNT_FIELD_NUMBER: _ClassVar[int]
    MIN_KEY_FIELD_NUMBER: _ClassVar[int]
    MAX_KEY_FIELD_NUMBER: _ClassVar[int]
    PART_COUNT_FIELD_NUMBER: _ClassVar[int]
    INDEX_TYPE_FIELD_NUMBER: _ClassVar[int]
    CURRENT_COUNT_FIELD_NUMBER: _ClassVar[int]
    DELETED_COUNT_FIELD_NUMBER: _ClassVar[int]
    MAX_ID_FIELD_NUMBER: _ClassVar[int]
    MIN_ID_FIELD_NUMBER: _ClassVar[int]
    MEMORY_BYTES_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    rows_count: int
    min_key: bytes
    max_key: bytes
    part_count: int
    index_type: _proxy_common_pb2.VectorIndexType
    current_count: int
    deleted_count: int
    max_id: int
    min_id: int
    memory_bytes: int
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., rows_count: _Optional[int] = ..., min_key: _Optional[bytes] = ..., max_key: _Optional[bytes] = ..., part_count: _Optional[int] = ..., index_type: _Optional[_Union[_proxy_common_pb2.VectorIndexType, str]] = ..., current_count: _Optional[int] = ..., deleted_count: _Optional[int] = ..., max_id: _Optional[int] = ..., min_id: _Optional[int] = ..., memory_bytes: _Optional[int] = ...) -> None: ...



import dingosdk

from dingodb.common.document_rep import (
    DocumentType,
    DocValue,
    Document,
    DocWithId,
    DocWithScore,
    AddResult,
    DocSearchResult,
    DocQueryResult,
    DocScanQueryResult,
    DocIndexMetricsResult,
    DocDeleteResult
)

from typing import List, Dict

document_type_to_scalar_type = {
    dingosdk.Type.kBOOL: DocumentType.BOOL,
    dingosdk.Type.kINT64: DocumentType.INT64,
    dingosdk.Type.kDOUBLE: DocumentType.DOUBLE,
    dingosdk.Type.kSTRING: DocumentType.STRING,
    dingosdk.Type.kBYTES: DocumentType.BYTES,
}


def document_doc_value_to_doc_value(
        document_doc_value: dingosdk.DocValue
):
    if document_doc_value.GetType() == dingosdk.Type.kINT64:
        return DocValue(DocumentType.INT64, document_doc_value.IntValue())
    elif document_doc_value.GetType() == dingosdk.Type.kDOUBLE:
        return DocValue(DocumentType.DOUBLE, document_doc_value.DoubleValue())
    elif document_doc_value.GetType() == dingosdk.Type.kSTRING:
        return DocValue(DocumentType.STRING, document_doc_value.StringValue())
    elif document_doc_value.GetType() == dingosdk.Type.kBYTES:
        return DocValue(DocumentType.BYTES, document_doc_value.BytesValue())
    elif document_doc_value.GetType() == dingosdk.Type.kBOOL:
        return DocValue(DocumentType.BOOL, document_doc_value.BoolValue())
    else:
        raise RuntimeError(f"not support type: {document_doc_value.GetType()}")


def document_document_to_document(
        document_document: dingosdk.Document
):
    fields_result = {}
    fields = document_document.GetFields()
    for key, value in fields.items():
        fields_result[key] = document_doc_value_to_doc_value(value)

    return Document(fields_result)


def document_doc_with_id_to_doc_with_id(
        document_doc_with_id: dingosdk.DocWithId
):
    return DocWithId(
        id=document_doc_with_id.id,
        doc=document_document_to_document(document_doc_with_id.doc)
    )


def document_doc_with_score_to_doc_with_score(
        document_doc_with_score: dingosdk.DocWithStore
):
    return DocWithScore(
        id=document_doc_with_score.doc_with_id.id,
        doc=document_document_to_document(document_doc_with_score.doc_with_id.doc),
        score=document_doc_with_score.score
    )


def document_add_result_to_add_result(
        document_add_result: List[dingosdk.DocWithId]
):
    return AddResult(
        docs=[
            document_doc_with_id_to_doc_with_id(x) for x in document_add_result
        ]
    )


def document_search_result_to_search_result(
        document_query_result: dingosdk.DocSearchResult
):
    return DocSearchResult(
        docs=[
            document_doc_with_score_to_doc_with_score(x)
            for x in document_query_result.doc_sores
        ]
    )




def document_scan_query_result_to_scan_query_result(
        document_scan_query_result: dingosdk.DocScanQueryResult
):
    return DocScanQueryResult(
        docs=[
            document_doc_with_id_to_doc_with_id(x)
            for x in document_scan_query_result.docs
        ]
    )


def document_get_index_metrics_to_get_index_metrics(
        document_get_index_metrics: dingosdk.DocIndexMetricsResult,
) -> DocIndexMetricsResult:
    return DocIndexMetricsResult(
        total_num_docs=document_get_index_metrics.total_num_docs,
        total_num_tokens=document_get_index_metrics.total_num_tokens,
        max_doc_id=document_get_index_metrics.max_doc_id,
        min_doc_id=document_get_index_metrics.min_doc_id,
        meta_json=document_get_index_metrics.meta_json,
        json_parameter=document_get_index_metrics.json_parameter
    )


def document_delete_result_to_delete_result(
        document_delete_result: dingosdk.DocDeleteResult,
) -> DocDeleteResult:
    return DocDeleteResult(
        doc_id=document_delete_result.doc_id,
        deleted=document_delete_result.deleted
    )



#!/usr/bin/python3

import dingosdk


class SDKParamFactory:
    def __init__(self):
        pass

    @staticmethod
    def create_flat_param(index_type: str, flat_param_dict: dict) -> dingosdk.FlatParam:
        assert index_type == "flat"

        return dingosdk.FlatParam(
            flat_param_dict["dimension"], flat_param_dict["metricType"]
        )

    @staticmethod
    def create_ivf_flat_param(
        index_type: str, ivf_flat_param_dict: dict
    ) -> dingosdk.IvfFlatParam:
        assert index_type == "ivf_flat"

        param = dingosdk.IvfFlatParam(
            ivf_flat_param_dict["dimension"],
            ivf_flat_param_dict["metricType"],
        )
        param.ncentroids = ivf_flat_param_dict["ncentroids"]

        return param

    @staticmethod
    def create_ivf_pq_param(
        index_type: str, ivf_pq_param_dict: dict
    ) -> dingosdk.IvfPqParam:
        assert index_type == "ivf_pq"

        param = dingosdk.IvfPqParam(
            ivf_pq_param_dict["dimension"],
            ivf_pq_param_dict["metricType"],
        )
        param.ncentroids = ivf_pq_param_dict["ncentroids"]
        param.nsubvector = ivf_pq_param_dict["nsubvector"]
        param.bucket_init_size = ivf_pq_param_dict["bucketInitSize"]
        param.bucket_max_size = ivf_pq_param_dict["bucketMaxSize"]
        param.nbits_per_idx = ivf_pq_param_dict["nbitsPerIdx"]

        return param

    @staticmethod
    def create_hnsw_param(index_type: str, hnsw_param_dict: dict) -> dingosdk.HnswParam:
        assert index_type == "hnsw"

        param = dingosdk.HnswParam(
            hnsw_param_dict["dimension"],
            hnsw_param_dict["metricType"],
            hnsw_param_dict["maxElements"],
        )
        param.ef_construction = hnsw_param_dict["efConstruction"]
        param.nlinks = hnsw_param_dict["nlinks"]

        return param

    @staticmethod
    def create_diskann_param(
        index_type: str, diskann_param_dict: dict
    ) -> dingosdk.DiskAnnParam:
        assert index_type == "diskann"

        param = dingosdk.DiskAnnParam(
            diskann_param_dict["dimension"],
            diskann_param_dict["metricType"],
            diskann_param_dict["valueType"],
        )

        param.search_list_size = diskann_param_dict["searchListSize"]
        param.max_degree = diskann_param_dict["maxDegree"]

        return param

    @staticmethod
    def create_brute_param(
        index_type: str, brute_param_dict: dict
    ) -> dingosdk.BruteForceParam:
        assert index_type == "brute"

        return dingosdk.BruteForceParam(
            brute_param_dict["dimension"],
            brute_param_dict["metricType"],
        )


    @staticmethod
    def create_binary_flat_param(index_type: str, binary_flat_param_dict: dict) -> dingosdk.BinaryFlatParam:
        assert index_type == "binary_flat"

        return dingosdk.BinaryFlatParam(
            binary_flat_param_dict["dimension"], binary_flat_param_dict["metricType"]
        )

    @staticmethod
    def create_binary_ivf_flat_param(
        index_type: str, binary_ivf_flat_param_dict: dict
    ) -> dingosdk.BinaryIvfFlatParam:
        assert index_type == "binary_ivf_flat"

        param = dingosdk.BinaryIvfFlatParam(
            binary_ivf_flat_param_dict["dimension"],
            binary_ivf_flat_param_dict["metricType"],
        )
        param.ncentroids = binary_ivf_flat_param_dict["ncentroids"]

        return param


#!/usr/bin/env python
#

import os

import setuptools

long_desc = """# dingodb

dingodb is a SDK for  vector database DingoDB, .
"""


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname), "r", encoding="utf-8") as fh:
        return fh.read()


setuptools.setup(
    name="dingodb",
    version=read("dingodb/__version__").strip(),
    description="dingodb is dingodb sdk",
    license="Proprietary License",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://www.dingodb.com/",
    project_urls={
        "Homepage": "https://www.dingodb.com/",
        "Documentation": "https://dingodb.readthedocs.io/en/latest/",
    },
    author="DingoDB",
    author_email="dingodb@zetyun.com",
    keywords="dingodb",
    packages=setuptools.find_packages(),
    install_requires=read("requirements.txt"),
    extras_require={
    },
    include_package_data=True,
    python_requires=">=3.7",
    entry_points={
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Intended Audience :: System Administrators",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Database",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]
)



from enum import Enum

from typing import List, Dict, Union


class DocumentType(Enum):
    BOOL = "BOOL"
    INT64 = "INT64"
    DOUBLE = "DOUBLE"
    STRING = "STRING"
    BYTES = "BYTES"


class DocumentColumn:
    def __init__(self, key: str, type: DocumentType) -> None:
        self.key = key
        self.type = type

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return {"key": self.key, "type": self.type}


class DocumentSchema:
    def __init__(self):
        self.cols = []

    def __str__(self):
        return str(self.to_dict())

    def add_document_column(self, col: DocumentColumn):
        self.cols.append(col)

    def to_dict(self):
        return [col.to_dict() for col in self.cols]


class DocValue:
    def __init__(
            self,
            type: DocumentType,
            value: Union[int, float, str]
    ):
        self.type = type
        self.value = value

    def __str__(self):
        return str(self.to_dict())


    def to_dict(self):
        return {"type": self.type.value, "value": self.value}


class Document:
    def __init__(
            self,
            fields: Dict[str, DocValue]
    ):
        self.fields = fields

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return {key: value.to_dict() for key, value in self.fields.items()}


class DocWithId:
    def __init__(
            self,
            id: int,
            doc: Document
    ):
        self.id = id
        self.doc = doc

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return {
            "id": self.id,
            "doc": self.doc.to_dict()
        }


class DocWithScore:
    def __init__(
            self,
            id: int,
            doc: Document,
            score: float
    ):
        self.id = id
        self.doc = doc
        self.score = score

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return {
            "id": self.id,
            "doc": self.doc.to_dict(),
            "score": self.score
        }


class AddResult:
    def __init__(
            self,
            docs: List[DocWithId]
    ):
        self.docs = docs

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return [doc.to_dict() for doc in self.docs]


class DocSearchResult:
    def __init__(
            self,
            docs: List[DocWithScore]
    ):
        self.docs = docs

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return [doc.to_dict() for doc in self.docs]


class DocQueryResult:
    def __init__(
            self,
            docs: List[Union[DocWithId, None]]
    ):
        self.docs = docs

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return [doc.to_dict() if doc is not None else None for doc in self.docs]


class DocScanQueryResult:
    def __init__(
            self,
            docs: List[DocWithId]
    ):
        self.docs = docs

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return [doc.to_dict() for doc in self.docs]


class DocIndexMetricsResult:
    def __init__(
            self,
            total_num_docs: int,
            total_num_tokens: int,
            max_doc_id: int,
            min_doc_id: int,
            meta_json: int,
            json_parameter: bool
    ):
        self.total_num_docs = total_num_docs
        self.total_num_tokens = total_num_tokens
        self.max_doc_id = max_doc_id
        self.min_doc_id = min_doc_id
        self.meta_json = meta_json
        self.json_parameter = json_parameter

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return {
            "total_num_docs": self.total_num_docs,
            "total_num_tokens": self.total_num_tokens,
            "max_doc_id": self.max_doc_id,
            "min_doc_id": self.min_doc_id,
            "meta_json": self.meta_json,
            "json_parameter": self.json_parameter
        }


class DocDeleteResult:
    def __init__(
            self,
            doc_id: int,
            deleted: bool
    ):
        self.doc_id = doc_id
        self.deleted = deleted

    def __str__(self):
        return str(self.to_dict())

    def to_dict(self):
        return {
            "doc_id": self.doc_id,
            "deleted": self.deleted
        }



from typing import Any, cast

from dingodb.utils.parser import Comparator, Comparison, Operation, Operator, get_parser

DEFAULT_PARSER = get_parser()

value_type_map = {
    int: "INT64",
    str: "STRING",
    float: "DOUBLE",
    bool: "BOOL",
    "unknown": "STRING",
}


def parser_json(actual):
    ori_dict = {}
    for key, value in actual.__dict__.items():
        if key == "operator" or key == "comparator":
            ori_dict["type"] = key
            ori_dict[key] = value.__dict__["_value_"]
        elif key == "arguments":
            ori_dict["arguments"] = []
            for arguments_value in value:
                ori_dict["arguments"].append(parser_json(arguments_value))
        elif key == "value":
            ori_dict[key] = value
            ori_dict["value_type"] = value_type_map.get(type(value), "STRING")
        else:
            ori_dict[key] = value
    return ori_dict


def auto_value_type(value):
    return value_type_map.get(type(value), "STRING")


def auto_expr_type(value):
    if value is None:
        return {}
    elif value.__class__.__name__ == 'Operation' or value.__class__.__name__ == 'Comparison':
        return parser_json(value)
    elif isinstance(value, str):
        return parser_json(DEFAULT_PARSER.parse(value))
    else:
        raise Exception(f"expr type must in [str,Operation, Comparison]")


def convert_dict_to_expr(d):
    conditions = []
    for key, value in d.items():
        condition = f'eq("{key}", "{value}")' if isinstance(value, str) else  f'eq("{key}", {value})'
        conditions.append(condition)
        
    return f'and({", ".join(conditions)})' if len(conditions)  > 1 else conditions[0]

if __name__ == "__main__":
    print(auto_value_type(1))
    print(auto_value_type("1"))
    print(auto_value_type(1.1))
    print(auto_value_type(True))
    print(auto_expr_type(f'eq("x", {100})'))
    print(
        auto_expr_type(
            'and(or(eq("a", "b"), eq("a", "c"), eq("a", "d")), not(eq("z", "foo")))'
        )
    )
    op = 'and(or(eq("a", "b"), eq("a", "c"), eq("a", "d")), not(eq("z", "foo")))'
    actual = DEFAULT_PARSER.parse(op)
    print(auto_expr_type(actual))
    op = f'eq("x", {100})'
    actual = DEFAULT_PARSER.parse(op)
    print(auto_expr_type(actual))



import dingosdk


class SDKClient:
    def __init__(self, addrs: str):
        """
        Args:
            addrs (str): coordinator addrs, try to use like 127.0.0.1:22001,127.0.0.1:22002,127.0.0.1:22003

        Raises:
            RuntimeError: return error
        """
        self.addrs = addrs
        self.s, self.dingosdk_client = dingosdk.Client.BuildAndInitLog(addrs)

        if not self.s.ok():
            raise RuntimeError(f"dongo client build fail: {self.s.ToString()}")



#!/usr/bin/python3

import warnings
from pydantic import BaseModel, validator
from typing import List

import dingosdk

index_types = {
    "flat": dingosdk.VectorIndexType.kFlat,
    "ivf_flat": dingosdk.VectorIndexType.kIvfFlat,
    "ivf_pq": dingosdk.VectorIndexType.kIvfPq,
    "hnsw": dingosdk.VectorIndexType.kHnsw,
    "diskann": dingosdk.VectorIndexType.kDiskAnn,
    "brute": dingosdk.VectorIndexType.kBruteForce,
    "binary_flat": dingosdk.VectorIndexType.kBinaryFlat,
    "binary_ivf_flat":dingosdk.VectorIndexType.kBinaryIvfFlat,
}

metric_types = {
    "euclidean": dingosdk.MetricType.kL2,
    "dotproduct": dingosdk.MetricType.kInnerProduct,
    "cosine": dingosdk.MetricType.kCosine,
    "hamming": dingosdk.MetricType.kHamming
}

value_type = {
    "float": dingosdk.ValueType.kFloat,
    "uint8": dingosdk.ValueType.kUint8,
    "int8": dingosdk.ValueType.kInt8,
}

index_params = {
    "flat": {"dimension": None, "metricType": None},
    "ivf_flat": {"dimension": None, "metricType": None, "ncentroids": 256},
    "ivf_pq": {
        "dimension": None,
        "metricType": None,
        "ncentroids": 256,
        "bucketInitSize": 1000,
        "bucketMaxSize": 128000,
        "nsubvector": 8,
        "nbitsPerIdx": 8,
    },
    "hnsw": {
        "dimension": None,
        "metricType": None,
        "efConstruction": 200,
        "maxElements": 50000,
        "nlinks": 32,
    },
    "diskann": {
        "dimension": None,
        "metricType": None,
        "valueType": None,
        "maxDegree": 64,
        "searchListSize": 100,
    },
    "brute": {"dimension": None, "metricType": None},
    "binary_flat":{"dimension": None, "metricType": None},
    "binary_ivf_flat": {"dimension": None, "metricType": None, "ncentroids": 256},
}


class CreateIndexParam(BaseModel):
    index_name: str
    dimension: int
    index_type: str
    metric_type: str
    replicas: int = 3
    index_config: dict = None
    metadata_config: dict = None
    partition_rule: dict = None
    operand: list = None
    auto_id: bool = True
    start_id: int = 1

    @validator("index_type", always=True)
    def check_index_type(cls, value):
        if value not in index_types.keys():
            raise Exception(f"index_type  must in {list(index_types.keys())}")
        return value

    @validator("metric_type", always=True)
    def check_metric_type(cls, value):
        if value not in metric_types.keys():
            raise Exception(f"metric_type  must in {metric_types.keys()}")
        return value

    @validator("replicas", always=True)
    def check_replicas(cls, value):
        if value < 0:
            raise Exception(f"{value} must >= 0")
        if value == 0:
            value = 3
        return value

    @validator("index_config", always=True)
    def check_index_config(cls, value, values):
        index_type = values.get("index_type")

        dimension = values.get("dimension")
        metric_type = values.get("metric_type")

        vector_index_parameter = index_params[index_type]

        if value is not None:
            for key, v in value.items():
                if key in vector_index_parameter.keys():
                    if key == "efConstruction":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 500 or v < 100:
                            warnings.warn(f"efConstruction: {v} suggestion in 100-500")
                    if key == "maxElements":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 1000000000 or v < 50000:
                            warnings.warn(
                                f"maxElements:{v} suggestion in 50000-1000000000"
                            )
                    if key == "nlinks":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 64 or v < 16:
                            warnings.warn(f"nlinks:{v} suggestion in 16-64")

                    if key == "nsubvector":
                        if dimension % v != 0:
                            raise Exception(f"dimension/nsubvector must Divisible")
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                        if v > 256 or v < 4:
                            warnings.warn(f"nsubvector:{v} suggestion in 4-256")
                    if key == "ncentroids":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                        if v > 4096 or v < 4:
                            warnings.warn(f"ncentroids:{v} suggestion in 4-4096")
                    if key == "bucketInitSize":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                    if key == "bucketMaxSize":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                    if key == "nbitsPerIdx":
                        if v <= 0 or v > 16:
                            raise Exception(f"{key} must > 0 and <=16")
                    if key == "maxDegree":
                        if v < 60 or v > 150:
                            raise Exception(f"{key} must >= 60 and <=150")
                    if key == "searchListSize":
                        if v < 75 or v > 200:
                            raise Exception(f"{key} must >= 75 and <=200")
                    if key == "valueType":
                        if v != "float":
                            raise Exception(f"{key} must == float")
                        vector_index_parameter[key] = value_type.get(v)
                        continue
                    vector_index_parameter[key] = v
                else:
                    warnings.warn(f"index_config {key} not in {index_type}")

        vector_index_parameter["dimension"] = dimension
        vector_index_parameter["metricType"] = metric_types[metric_type]

        return vector_index_parameter

    @validator("partition_rule", always=True)
    def check_partition_rule(cls, value):
        value = {} if value is None else value
        if len(value) != 0:
            raise RuntimeError("partition_rule is not support now")
        return value

    @validator("metadata_config", always=True)
    def check_metadata_config(cls, value):
        value = {} if value is None else value
        if len(value) != 0:
            raise RuntimeError("metadata_config is not support now")

    @validator("start_id", always=True)
    def check_start_id(cls, value, values):
        auto_id = values.get("auto_id")
        if auto_id:
            if value < 1:
                raise Exception(f"start_id must >= 1")
        return value


class VectorAddParam(BaseModel):
    index_name: str
    vectors: List[list]
    datas: List[dict] = None
    ids: List[int] = None
    value_type: str = "float"

    @validator("value_type", always=True)
    def check_value_type(cls, value, values):
        if value is None:
            value = "float"  
        if value not in {"binary", "float"}:
            raise Exception("value_type must be 'binary' or 'float'")
        return value

    @validator("datas", always=True)
    def check_datas(cls, value, values):
        value = [{}] * len(values.get("vectors")) if value is None else value
        return value

    @validator("ids", always=True)
    def check_ids(cls, value, values):
        if value is None:
            assert len(values.get("datas")) == len(
                values.get("vectors")
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} is not equal"
        else:
            assert (
                len(values.get("datas")) == len(values.get("vectors")) == len(value)
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} ids:{len(value)} is not equal"
            for id in value:
                if id <= 0:
                    raise Exception("id must > 0")
        return value

class VectorScanParam(BaseModel):
    index_name: str
    start_id: int
    max_count: int = 1000
    is_reverse: bool = False
    with_scalar_data: bool = True
    with_table_data: bool = True
    with_vector_data: bool = True
    fields: list = None
    filter_scalar: dict = None
    end_id: int = 0

    @validator("end_id", pre=True, always=True)
    def check_true_positive_values(cls, value, field):
        if not value >= 0:
            raise ValueError(f"{field.name} must >= 0")
        return value

    @validator("start_id", "max_count", pre=True, always=True)
    def check_positive_values(cls, value, field):
        if not value > 0:
            raise ValueError(f"{field.name} must > 0")
        return value

    @validator("fields", pre=True, always=True)
    def check_fields(cls, value):
        return value or []

    @validator("filter_scalar", pre=True, always=True)
    def check_filter_scalar(cls, value):
        if value is None:
            return {}
        else:
            return value


class VectorSearchParam(BaseModel):
    index_name: str
    xq: list
    top_k: int
    fields: list = None  # select keys
    pre_filter: bool = True
    brute: bool = False
    search_params: dict = None
    with_vector_data: bool = True
    with_scalar_data: bool = True
    beamwidth: int = 2
    value_type: str="float"

    @validator("value_type", always=True)
    def check_value_type(cls, value, values):
        if value is None:
            value = "float"  
        if value not in {"binary", "float"}:
            raise Exception("value_type must be 'binary' or 'float'")
        return value

    @validator("xq", always=True)
    def check_xq(cls, value):
        if not isinstance(value[0], list):
            value = [value]
        return value

    @validator("top_k", pre=True, always=True)
    def check_top_k(cls, value, field):
        if not value > 0:
            raise ValueError(f"{field.name} must > 0")
        return value

    @validator("fields", always=True)
    def check_fields(cls, value):
        if value is None:
            value = []
        return value

    @validator("search_params", always=True)
    def check_search_params(cls, search_params, values):
        if search_params is None or len(search_params) == 0:
            search_params = {}
            values["pre_filter"] = False
            return search_params

        assert search_params is not None, "search_params is None"

        values["with_vector_data"] = search_params.get("withVectorData", True)
        values["with_scalar_data"] = search_params.get("withScalarData", True)

        if (
            "meta_expr" in search_params.keys()
            and "langchain_expr" in search_params.keys()
        ):
            raise ValueError(f"meta_expr and langchain_expr cannot coexist")

        if (
            "meta_expr" not in search_params.keys()
            and "langchain_expr" not in search_params.keys()
        ):
            values["pre_filter"] = False

        return search_params


class VectorGetParam(BaseModel):
    index_name: str
    ids: List[int]
    with_scalar_data: bool = True
    with_vector_data: bool = True

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)

        return id_list


class VectorDeleteParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        return id_list


class VectorStatusByRegionIdParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        return id_list


class VectorBuildByRegionIdParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        return id_list


class VectorLoadByRegionIdParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        return id_list


class VectorResetByRegionIdParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        return id_list
